/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : r_sdadc24_control_api.h
* Version      : 1.0
* Description  : SDADC24 control header file
***********************************************************************************************************************/
/***********************************************************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 30.04.2020 1.00     First Release
***********************************************************************************************************************/

#ifndef R_SDADC24_CONTROL_API_H_
#define R_SDADC24_CONTROL_API_H_

/***********************************************************************************************************************
 Includes   <System Includes> , "Project Includes"
 **********************************************************************************************************************/
#include "hal_data.h"

/***********************************************************************************************************************
 Macro definitions
 **********************************************************************************************************************/
/** Set ADC Channel number. Set any of ADC_CHANNEL_0 to ADC_CHANNEL_3. */
#define D_SDADC24_WEIGHT_CHANNEL    (ADC_CHANNEL_0)

/** Set gain. Set any of E_PGA_SDADC24_GAIN_1_1_1 to E_PGA_SDADC24_GAIN_8_4_32. */
#define D_SDADC24_GAIN              (E_PGA_SDADC24_GAIN_8_4_32)

/** Set offset voltage. Set any of E_PGA_SDADC24_OFFSET_M164P06 to E_PGA_SDADC24_OFFSET_164P06. */
#define D_SDADC24_OFFSET            (E_PGA_SDADC24_OFFSET_0P00)

/** Number of A/D conversions for disconnect detect.   */
#define D_SDADC24_DISCONN_NUM       (10U)

/** SDADC24 overflow threshold.                        */
#define D_SDADC24_OVERFLOW_THRESH   (0x01000000L)

/***********************************************************************************************************************
 Global Typedef definitions
 **********************************************************************************************************************/
/** Gain setting specification enumerator */
typedef enum
{
    E_PGA_SDADC24_GAIN_1_1_1        = 0x00U,                    /* 0b00000 : GSET1 = 1, GSET2 = 1, GTOTAL = 1         */
    E_PGA_SDADC24_GAIN_2_1_2        = 0x04U,                    /* 0b00100 : GSET1 = 2, GSET2 = 1, GTOTAL = 2         */
    E_PGA_SDADC24_GAIN_3_1_3        = 0x08U,                    /* 0b01000 : GSET1 = 3, GSET2 = 1, GTOTAL = 3         */
    E_PGA_SDADC24_GAIN_4_1_4        = 0x0CU,                    /* 0b01100 : GSET1 = 4, GSET2 = 1, GTOTAL = 4         */
    E_PGA_SDADC24_GAIN_8_1_8        = 0x10U,                    /* 0b10000 : GSET1 = 8, GSET2 = 1, GTOTAL = 8         */
    E_PGA_SDADC24_GAIN_1_2_2        = 0x01U,                    /* 0b00001 : GSET1 = 1, GSET2 = 2, GTOTAL = 2         */
    E_PGA_SDADC24_GAIN_2_2_4        = 0x05U,                    /* 0b00101 : GSET1 = 2, GSET2 = 2, GTOTAL = 4         */
    E_PGA_SDADC24_GAIN_3_2_6        = 0x09U,                    /* 0b01001 : GSET1 = 3, GSET2 = 2, GTOTAL = 6         */
    E_PGA_SDADC24_GAIN_4_2_8        = 0x0DU,                    /* 0b01101 : GSET1 = 4, GSET2 = 2, GTOTAL = 8         */
    E_PGA_SDADC24_GAIN_8_2_16       = 0x11U,                    /* 0b10001 : GSET1 = 8, GSET2 = 2, GTOTAL = 16        */
    E_PGA_SDADC24_GAIN_1_4_4        = 0x02U,                    /* 0b00010 : GSET1 = 1, GSET2 = 4, GTOTAL = 4         */
    E_PGA_SDADC24_GAIN_2_4_8        = 0x06U,                    /* 0b00110 : GSET1 = 2, GSET2 = 4, GTOTAL = 8         */
    E_PGA_SDADC24_GAIN_3_4_12       = 0x0AU,                    /* 0b01010 : GSET1 = 3, GSET2 = 4, GTOTAL = 12        */
    E_PGA_SDADC24_GAIN_4_4_16       = 0x0EU,                    /* 0b01110 : GSET1 = 4, GSET2 = 4, GTOTAL = 16        */
    E_PGA_SDADC24_GAIN_8_4_32       = 0x12U,                    /* 0b10010 : GSET1 = 8, GSET2 = 4, GTOTAL = 32        */
    E_PGA_SDADC24_GAIN_1_8_8        = 0x03U,                    /* 0b00011 : GSET1 = 1, GSET2 = 8, GTOTAL = 8         */
    E_PGA_SDADC24_GAIN_2_8_16       = 0x07U,                    /* 0b00111 : GSET1 = 2, GSET2 = 8, GTOTAL = 16        */
    E_PGA_SDADC24_GAIN_3_8_24       = 0x0BU,                    /* 0b01011 : GSET1 = 3, GSET2 = 8, GTOTAL = 24        */
    E_PGA_SDADC24_GAIN_4_8_32       = 0x0FU,                    /* 0b01111 : GSET1 = 4, GSET2 = 8, GTOTAL = 32        */
} e_pga_sdadc24_gain_t;

/** Offset voltage setting specification enumerator */
typedef enum
{
    E_PGA_SDADC24_OFFSET_M164P06    = 0x11U,                    /* 0b10001 : -164.06 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M153P13    = 0x12U,                    /* 0b10010 : -153.13 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M142P19    = 0x13U,                    /* 0b10011 : -142.19 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M131P25    = 0x14U,                    /* 0b10100 : -131.25 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M120P31    = 0x15U,                    /* 0b10101 : -120.31 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M109P38    = 0x16U,                    /* 0b10110 : -109.38 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M98P44     = 0x17U,                    /* 0b10111 :  -98.44 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M87P50     = 0x18U,                    /* 0b11000 :  -87.50 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M76P56     = 0x19U,                    /* 0b11001 :  -76.56 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M65P63     = 0x1AU,                    /* 0b11010 :  -65.63 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M54P69     = 0x1BU,                    /* 0b11011 :  -54.69 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M43P75     = 0x1CU,                    /* 0b11100 :  -43.75 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M32P81     = 0x1DU,                    /* 0b11101 :  -32.81 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M21P88     = 0x1EU,                    /* 0b11110 :  -21.88 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_M10P94     = 0x1FU,                    /* 0b11111 :  -10.94 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_0P00       = 0x00U,                    /* 0b00000 :       0 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_10P94      = 0x01U,                    /* 0b00001 :   10.94 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_21P88      = 0x02U,                    /* 0b00010 :   21.88 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_32P81      = 0x03U,                    /* 0b00011 :   32.81 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_43P75      = 0x04U,                    /* 0b00100 :   43.75 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_54P69      = 0x05U,                    /* 0b00101 :   54.69 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_65P63      = 0x06U,                    /* 0b00110 :   65.63 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_76P56      = 0x07U,                    /* 0b00111 :   76.56 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_87P50      = 0x08U,                    /* 0b01000 :   87.50 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_98P44      = 0x09U,                    /* 0b01001 :   98.44 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_109P38     = 0x0AU,                    /* 0b01010 :  109.38 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_120P31     = 0x0BU,                    /* 0b01011 :  120.31 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_131P25     = 0x0CU,                    /* 0b01100 :  131.25 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_142P19     = 0x0DU,                    /* 0b01101 :  142.19 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_153P13     = 0x0EU,                    /* 0b01110 :  153.13 / GSET1 [mV]                     */
    E_PGA_SDADC24_OFFSET_164P06     = 0x0FU,                    /* 0b01111 :  164.06 / GSET1 [mV]                     */
} e_pga_sdadc24_offset_t;

/** Gain setting structure */
typedef struct
{
    uint8_t gset1;
    uint8_t gset2;
    uint8_t gtotal;
} st_gain_setting_t;

/***********************************************************************************************************************
 Exported global variables
 **********************************************************************************************************************/
extern sdadc_instance_ctrl_t    g_sdadc24_ctrl;
extern sdadc_extended_cfg_t     g_sdadc24_extend_cfg;
extern adc_cfg_t                g_sdadc24_cfg;
extern adc_instance_t           g_sdadc24;
extern sdadc_scan_cfg_t         g_sdadc24_channel_cfg;
extern sdadc_channel_cfg_t      g_sdadc24_channel_cfgs;

/***********************************************************************************************************************
 Exported global functions (to be accessed by other files)
 **********************************************************************************************************************/

/***********************************************************************************************************************
 * Function Name: R_SDADC24_Init
 * Description  : This function set a value to the variables of SDADC24.
 * Arguments    : adc_cfg_t *p_cfg
 *                  ADC configuration structure pointer.
 *                sdadc_scan_cfg_t *p_channel_cfg
 *                  ADC scan configuration structure pointer.
 *                sdadc_channel_cfg_t *p_channel_cfgs
 *                  ADC channel configuration structure pointer.
 *                sdadc_extended_cfg_t *p_extended_cfg
 *                  ADC extended configuration structure pointer.
 * Return Value : None
 **********************************************************************************************************************/
void R_SDADC24_Init (adc_cfg_t *p_cfg,                    sdadc_scan_cfg_t *p_channel_cfg,
                     sdadc_channel_cfg_t *p_channel_cfgs, sdadc_extended_cfg_t *p_extended_cfg);

/***********************************************************************************************************************
 * Function Name: R_SDADC24_ConfigDisconnectDetection
 * Description  : This function set a value to the variables of SDADC24.
 * Arguments    : adc_cfg_t *p_cfg
 *                  ADC configuration structure pointer.
 *                sdadc_scan_cfg_t *p_channel_cfg
 *                  ADC scan configuration structure pointer.
 *                sdadc_channel_cfg_t *p_channel_cfgs
 *                  ADC channel configuration structure pointer.
 *                sdadc_extended_cfg_t *p_extended_cfg
 *                  ADC extended configuration structure pointer.
 *                uint8_t polarity
 *                  ADC channel polarity. (0: Positive, 1: Negative)
 * Return Value : None
 **********************************************************************************************************************/
void R_SDADC24_ConfigDisconnectDetection (adc_cfg_t *p_cfg,                    sdadc_scan_cfg_t *p_channel_cfg,
                                          sdadc_channel_cfg_t *p_channel_cfgs, sdadc_extended_cfg_t *p_extended_cfg,
                                          uint8_t polarity);

/***********************************************************************************************************************
 * Function Name: R_SDADC24_ScanStart
 * Description  : This function calibrate, set offset voltage and start continuous scan.
 * Arguments    : adc_ctrl_t *p_instance
 *                  ADC control block pointer
 *                int32_t offset_reg_val
 *                  Value set in the PGAOFS bit of the PGACn register.
 * Return Value : fsp_err_t
 **********************************************************************************************************************/
fsp_err_t R_SDADC24_ScanStart (adc_ctrl_t *p_instance, int32_t offset_reg_val);

/***********************************************************************************************************************
 * Function Name: R_SDADC24_DisconnectDetectSinglePolarity
 * Description  : This function set SDADC24 offset adjustment.
 * Arguments    : adc_ctrl_t *p_instance
 *                  ADC control block pointer.
 *                bool *p_result
 *                  Pointer to the disconnect check result storage area.
 *                  true: Connected, false: Disconnected
 * Return Value : fsp_err_t
 **********************************************************************************************************************/
fsp_err_t R_SDADC24_DisconnectDetectSinglePolarity (adc_ctrl_t *p_instance, bool *p_result);

/***********************************************************************************************************************
 * Function Name: R_SDADC24_DisconnectDetect
 * Description  : This function disconnect detection.
 *                Check for positive polarity and then for negative polarity.
 * Arguments    : adc_ctrl_t *p_instance
 *                  ADC control block pointer.
 *                adc_cfg_t *p_cfg
 *                  ADC configuration structure pointer.
 *                sdadc_scan_cfg_t *p_channel_cfg
 *                  ADC scan configuration structure pointer.
 *                sdadc_channel_cfg_t *p_channel_cfgs
 *                  ADC channel configuration structure pointer.
 *                sdadc_extended_cfg_t *p_extended_cfg
 *                  ADC extended configuration structure pointer.
 *                bool *p_result
 *                  Pointer to the disconnect detection result storage area.
 *                  true: Connected, false: Disconnected
 * Return Value : None
 **********************************************************************************************************************/
fsp_err_t R_SDADC24_DisconnectDetect (adc_ctrl_t *p_instance,               adc_cfg_t *p_cfg,
                                      sdadc_scan_cfg_t *p_channel_cfg,      sdadc_channel_cfg_t *p_channel_cfgs,
                                      sdadc_extended_cfg_t *p_extended_cfg, bool *p_result);

/***********************************************************************************************************************
 * Function Name: R_SDADC24_GetGain
 * Description  : This function get PGA gain setting.
 * Arguments    : st_gain_setting_t *p_gain.
 *                  Gain setting structure pointer.
 *                adc_channel_t adc_channel.
 *                  Target channel
 * Return Value : None
 **********************************************************************************************************************/
void R_SDADC24_GetGain (st_gain_setting_t *p_gain, adc_channel_t adc_channel);

/***********************************************************************************************************************
 * Function Name: R_SDADC24_GetOffset
 * Description  : This function get offset voltage (converted value to A/D value).
 * Arguments    : adc_channel_t adc_channel.
 *                  Target channel.
 * Return Value : int32_t
 *                  Offset voltage (converted value to A/D value)
 **********************************************************************************************************************/
int32_t R_SDADC24_GetOffset (adc_channel_t adc_channel);

/***********************************************************************************************************************
 * Function Name: R_SDADC24_OffsetAdjust
 * Description  : This function set SDADC24 offset adjustment.
 * Arguments    : adc_ctrl_t *p_instance
 *                  ADC control block pointer.
 *                adc_cfg_t *p_cfg
 *                  ADC configuration structure pointer.
 * Return Value : fsp_err_t
 **********************************************************************************************************************/
fsp_err_t R_SDADC24_OffsetAdjust (adc_ctrl_t *p_instance, int32_t *p_offset);

#endif /* R_SDADC24_CONTROL_API_H_ */
