/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : r_usb_control_api.h
* Version      : 1.0
* Description  : USB control header file
***********************************************************************************************************************/
/***********************************************************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 30.04.2020 1.00     First Release
***********************************************************************************************************************/

#ifndef R_USB_CONTROL_API_H_
#define R_USB_CONTROL_API_H_

/**********************************************************************************************************************
 Includes   <System Includes> , "Project Includes"
 *********************************************************************************************************************/

/**********************************************************************************************************************
 Macro definitions
 *********************************************************************************************************************/
#define D_LINE_CODING_LENGTH          (0x07U)
#define D_NUM_STRING_DESCRIPTOR       (0x07U)
#define D_USB_EP_PACKET_SIZE          (64U)

/**********************************************************************************************************************
 Global Typedef definitions
 *********************************************************************************************************************/
/** USB control structure */
typedef struct
{
    usb_instance_ctrl_t     *p_ctrl;
    usb_status_t            *p_status;
    usb_setup_t             *p_setup;
    usb_pcdc_linecoding_t   *p_linecoding;
    uint8_t                 *p_readbuf;
    uint8_t                 readbuf_size;
} st_usbctrl_ctrl_t;

/**********************************************************************************************************************
 Exported global variables
 *********************************************************************************************************************/
/** USB descriptor variables */
extern uint8_t                  g_apl_device[];
extern uint8_t                  g_apl_configuration[];
extern uint8_t                  g_apl_hs_configuration[];
extern uint8_t                  g_apl_qualifier_descriptor[];
extern uint8_t                  *g_apl_string_table[];
extern const usb_descriptor_t   g_usb_descriptor;           /* USB descriptor               */

/**********************************************************************************************************************
 Exported global functions (to be accessed by other files)
 *********************************************************************************************************************/

/**********************************************************************************************************************
 * Function Name: R_USBCTRL_EventPoll
 * Description  : This function processing according to USB event.
 * Arguments    : st_usbctrl_ctrl_t *p_usbctrl_ctrl
 *                  USB control structure pointer
 * Return Value : None
 *********************************************************************************************************************/
fsp_err_t R_USBCTRL_EventPoll (st_usbctrl_ctrl_t * const p_usbctrl_ctrl);

/**********************************************************************************************************************
 * Function Name: R_USBCTRL_Write
 * Description  : This function start sending data.
 * Arguments    : st_usbctrl_ctrl_t *p_usbctrl_ctrl
 *                  USB control structure pointer
 *                uint8_t *p_buf
 *                  Send data pointer
 *                uint32_t size
 *                  Size of send data
 *                uint8_t destination
 *                  USB class type
 * Return Value : None
 *********************************************************************************************************************/
fsp_err_t R_USBCTRL_Write (st_usbctrl_ctrl_t * const p_usbctrl_ctrl, uint8_t const * const p_buf, uint32_t size,
                           uint8_t destination);

/**********************************************************************************************************************
 * Function Name: R_USBCTRL_GetWriteStatus
 * Description  : This function get USB write status.
 * Arguments    : None
 * Return Value : bool
 *                  true:Complete, false:Incomplete
 *********************************************************************************************************************/
bool R_USBCTRL_GetWriteStatus (void);

#endif /* R_USB_CONTROL_API_H_ */
